﻿/*  Copyright 2007 Philip D. Bober
    REMOTE ETHER ARACHNID WARTIME is distributed under the terms of the GNU General Public License.

    This file is part of REMOTE ETHER ARACHNID WARTIME.

    REMOTE ETHER ARACHNID WARTIME is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    REMOTE ETHER ARACHNID WARTIME is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with REMOTE ETHER ARACHNID WARTIME.  If not, see <http://www.gnu.org/licenses/>.
*/
package {
	import flash.display.*;
	import flash.geom.*;
	import flash.events.*;
	
	public class Enemy extends BitmapSprite {
		public static const ENEMY_TYPES=2;
		private var level:int=0;
		private var exactLevel:Number=0;
		private var path:DivePath=null;
		public var idling:Boolean=false;
		public var row:uint;
		public var savedPos:Point;
		public var returning:Boolean;
		private var lastTime:uint;
		public var game:Game;
		public function Enemy(game:Game,type:int,row:uint=0){
			this.game=game;
			initBitmap(type);
			this.row=row;
			returning=false;
			game.addLocalEventListener(this,Event.ENTER_FRAME, nextFrame);
		}
		private function initBitmap(type:int){
			switch(type){
				case 0:
					surf=new Bitmap(new bug1(24,28));
					surf.width=24;
					surf.height=28;
					break;
				case 1:
					surf=new Bitmap(new bug2(28,24));
					surf.width=28;
					surf.height=24;
					break;
			}
			surf.x=-surf.width/2;
			surf.y=-surf.height/2;
			addChild(surf);
			buildHitRect();
		}
		private function nextFrame(ev:Event):void{
			var now:uint=Time.get();
			var diff:uint=now-lastTime;
			lastTime=now;
			if(path!=null){
				path.update(diff);
				path.setPos(this);
				if(returning && path.ended){
					startIdle(true);
				}
			}else{
				exactLevel+=diff/20.0;
				level=int(exactLevel);
				rotation=level;
			}
		}
		public function enterPath(path:DivePath,idle:Boolean){
			this.path=path;
			this.path.setInitPos(this);
			this.level=0;
			this.idling=idle;
			this.returning=false;
		}
		public function onDive():Boolean{
			return !this.idling && this.path!=null;
		}
		public function diveEnded():Boolean{
			return this.path!=null && this.path.ended;
		}
		public function setPos(x:Number,y:Number):void{
			this.x=x;
			this.y=y;
			this.savedPos=new Point(x,y);
		}
		public function startIdle(needHelper:Boolean=false):void{
			var p:DivePath=new DivePath(DivePathData.idle);
			
			p.looping=true;
			this.enterPath(p,true);
			p.speed=0.5;
			var helper:Enemy;
			if(needHelper){
				helper=game.findHelper(this);
			}
			if(helper!=null){
				//trace("FOUND HELPER");
				p.setRelativeOffset(helper.path.getRelativeOffset());
			}else{
				//trace("NO HELPER :(");
				p.setRelativeOffset((row%2)/2.0);
			}
		}
	}
}