﻿/*  Copyright 2007 Philip D. Bober
    REMOTE ETHER ARACHNID WARTIME is distributed under the terms of the GNU General Public License.

    This file is part of REMOTE ETHER ARACHNID WARTIME.

    REMOTE ETHER ARACHNID WARTIME is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    REMOTE ETHER ARACHNID WARTIME is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with REMOTE ETHER ARACHNID WARTIME.  If not, see <http://www.gnu.org/licenses/>.
*/
package {
	import flash.display.*;
	import flash.geom.*;
	import flash.events.*;
	public class Game extends Mode {
		public var player:Player;
		private var enemies:Array=[];
		private var explosions:Array=[];
		private var enemyBullets:Array=[];
		public var diver:DiveManager;
		public var stars:Starfield;
		private var cheat:CheatManager;
		private var messages:Array=[];
		private var levels:LevelManager;
		public function Game (switcher:ModeSwitcher,stage:Stage) {
			super(switcher);
			levels=new LevelManager(this);
			GameData.levelReset();
			cheat=new CheatManager(this);
			addLocalEventListener(stage,KeyboardEvent.KEY_UP,hitKey);
			stars=new Starfield(stage);
			addChild(stars);
			initEnemies();
			player=new Player(this,stage);
			addChild(player);
			addLocalEventListener(this,Event.ENTER_FRAME, mainLoop);
			//addEventListener(false,0,true);
		}
		public function initEnemies():void{
			var i:int;
			for(i=0;i<enemies.length;i++){
				removeChild(enemies[i]);
			}
			enemies=levels.initLevel(GameData.currentLevel);
			for(i=0;i<enemies.length;i++){
				addChild(enemies[i]);
			}
			GameData.enemyCount=enemies.length;
			idleEveryEnemy();
			diver=new DiveManager(this,enemies);
			
		}
		public function addBigExplosion(x:int,y:int,level:int){
			addExplosion(x,y,0);
			for(var i=1;i<level;i++){
				var mx:int=x+(Math.random()-0.5)*30;
				var my:int=y+(Math.random()-0.5)*30;
				addExplosion(mx,my,i*100).setRotation(int(Math.random()*4));
			}
		}
		public function addExplosion(x:int,y:int,precreate:int=0):Explosion{
			var e:Explosion=new Explosion(x,y,precreate);
			addChild(e);
			explosions.push(e);
			return e;
		}
		public function mainLoop(ev:Event):void{
			var b,e;
			stars.update();
			for(e=0;e<explosions.length;e++){
				explosions[e].update();
			}
			for(b=0;b<player.bullets.length;b++){
				if(player.bullets[b].dead)continue;
				for(e=0;e<enemies.length;e++){
					if(enemies[e].hitTestObject(player.bullets[b])){
						addExplosion(enemies[e].x,enemies[e].y);
						GameData.enemyDead++;
						removeChild(enemies[e]);
						enemies.splice(e,1);
						e--;
						player.bullets[b].dead=true;
						removeChild(player.bullets[b]);
						player.bullets.splice(b,1);
						b--;
						GameData.hitCount++;
						break;
					}
				}
			}
			
			for(e=0;e<enemies.length;e++){
				if(enemies[e].hitTestObject(player)){
					addExplosion(enemies[e].x,enemies[e].y);
					killPlayer();
					GameData.enemyDead++;
					removeChild(enemies[e]);
					enemies.splice(e,1);
					e--;

				}
			}
			for(e=0;e<enemyBullets.length;e++){
				if(enemyBullets[e].hitTestObject(player)){
					addExplosion(enemyBullets[e].x,enemyBullets[e].y);
					killPlayer();
					removeChild(enemyBullets[e]);
					enemyBullets.splice(e,1);
					e--;

				}
			}
			if(enemies.length==0 && !player.dead){
				startWin();
			}
			for(e=0;e<messages.length;e++){
				if(messages[e].dead){
					removeChild(messages[e]);
					messages.splice(e,1);
					e--;
				}
			}
		}
		public function killPlayer():void{
			if(player.dead)return;//No point in killing the player twice
			if(player.godmode)return; //You cheating bastard
			GameData.endTime=Time.getReal();
			player.dead=true;
			addBigExplosion(player.x,player.y,6);
			simpleSchedule(200,removePlayerSprite);
			simpleSchedule(3000,endGame);
		}

		private function removePlayerSprite(event:TimerEvent):void{
			removeChild(player);
		}
		private function winLevel(event:Event=null):void{
			GameData.nextLevel=true;
			GameData.wonLevel=true;
			if(levels.nextLast(GameData.currentLevel)){
				GameData.wonGame=true;
				return winGame();
			}
			this.switcher.simpleMode(Mode.MODE_STATS);
		}
		public function endGame(event:TimerEvent=null):void{
			this.switcher.simpleMode(Mode.MODE_GAMEOVER);
		}
		public function winGame(event:TimerEvent=null):void{
			this.switcher.simpleMode(Mode.MODE_WIN);
		}
		public function startWin():void{
			player.flyOff();
			simpleSchedule(2500,winLevel);
			GameData.endTime=Time.getReal();
			GameData.wonLevel=true;
		}

		private function idleEveryEnemy():void{
			for(var i=0;i<enemies.length;i++){
				enemies[i].startIdle();
			}
		}
		public function findHelper(req:Enemy):Enemy{
			for(var i=0;i<enemies.length;i++){
				var e:Enemy=enemies[i];
				if(e!=req && e.row==req.row && e.idling && !e.returning){
					return e;
				}
			}
			return null;
		}
		public function createEnemyBullet(x:Number,y:Number){
			var b:Bullet = new Bullet(this,true);
			b.x=x-3;
			b.y=y-3;
			enemyBullets.push(b);
			addChild(b);
		}
		public function addMessage(str:String):void{
			var m:FloatingMessage=new FloatingMessage(this,str);
			this.messages.push(m);
			this.addChild(m);
		}
		private function hitKey(ev:KeyboardEvent):void{
			cheat.addInput(ev);
			if(ev.keyCode==27){ // ESCAPE
				switcher.simpleMode(Mode.MODE_MENU);
			}
		}
	}
}